/**
 * Maintained common support APIs provided by the JUnit Platform.
 *
 * <p>The purpose of this package is to provide {@code TestEngine} and
 * {@code Extension} authors convenient access to a subset of internal utility
 * methods to assist with their implementation. This prevents re-inventing the
 * wheel and ensures that common tasks are handled in third-party engines and
 * extensions with the same semantics as within the JUnit Platform itself.
 */

package org.junit.platform.commons.support;
