/**
 * Common APIs and support utilities for the JUnit Platform.
 *
 * <h3>DISCLAIMER</h3>
 *
 * <p>Any API annotated with {@code @API(status = INTERNAL)} is intended solely
 * for usage within the JUnit framework itself. <strong>Any usage of internal
 * APIs by external parties is not supported!</strong>
 */

package org.junit.platform.commons;
