/**
 * Core support classes for the {@link org.junit.platform.launcher.Launcher Launcher}
 * including the {@link org.junit.platform.launcher.core.LauncherFactory LauncherFactory}
 * and the {@link org.junit.platform.launcher.core.LauncherDiscoveryRequestBuilder
 * LauncherDiscoveryRequestBuilder}.
 */

package org.junit.platform.launcher.core;
