open module integration {
	requires org.junit.jupiter.api;
	requires org.junit.jupiter.engine;
	requires org.junit.platform.commons;
	requires org.junit.platform.engine;
	requires org.junit.platform.launcher;
	requires org.junit.platform.reporting;
}
