/*
 * This Java source file was generated by the Gradle 'init' task.
 */
package com.example.vintage;

import org.junit.Test;
import static org.junit.Assert.*;

public class VintageTest {
    @Test
    public void success() {
        // pass
    }
    @Test
    public void failure() {
        fail("expected to fail");
    }
}
