/**
 * Configurable test discovery implementation that can be reused by different test engines.
 *
 * @see org.junit.platform.engine.support.discovery.EngineDiscoveryRequestResolver#builder()
 * @see org.junit.platform.engine.support.discovery.EngineDiscoveryRequestResolver#resolve
 * @see org.junit.platform.engine.support.discovery.SelectorResolver
 */

package org.junit.platform.engine.support.discovery;
