/**
 * Support for generating XML reports using a format which is compatible with
 * the de facto standard for JUnit 4 based test reports that was made popular
 * by the Ant build system.
 */

package org.junit.platform.reporting.legacy.xml;
