/**
 * Support classes and base implementation for any
 * {@link org.junit.platform.engine.TestEngine} that wishes to organize test suites
 * hierarchically based on the
 * {@link org.junit.platform.engine.support.hierarchical.Node} abstraction.
 */

package org.junit.platform.engine.support.hierarchical;
