/**
 * Internal <em>logging</em> package.
 *
 * <h3>DISCLAIMER</h3>
 *
 * <p>These classes are intended solely for usage within the JUnit framework
 * itself. <strong>Any usage by external parties is not supported.</strong>
 * Use at your own risk!
 */

package org.junit.platform.commons.logging;
